/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o
{
	/// <exclude></exclude>
	public class YapMemoryFile : com.db4o.YapFile
	{
		private bool i_closed = false;

		internal readonly com.db4o.ext.MemoryFile i_memoryFile;

		private int i_length = 0;

		protected YapMemoryFile(com.db4o.YapStream a_parent, com.db4o.ext.MemoryFile memoryFile
			) : base(a_parent)
		{
			this.i_memoryFile = memoryFile;
			try
			{
				Open();
			}
			catch (System.Exception e)
			{
				com.db4o.inside.Exceptions4.ThrowRuntimeException(22, e);
			}
			Initialize3();
		}

		internal YapMemoryFile(com.db4o.ext.MemoryFile memoryFile) : this(null, memoryFile
			)
		{
		}

		public override void Backup(string path)
		{
			com.db4o.inside.Exceptions4.ThrowRuntimeException(60);
		}

		internal virtual void CheckDemoHop()
		{
		}

		internal override bool Close2()
		{
			i_entryCounter++;
			try
			{
				Write(true);
			}
			catch (System.Exception t)
			{
				FatalException(t);
			}
			base.Close2();
			i_entryCounter--;
			if (i_closed == false)
			{
				byte[] temp = new byte[i_length];
				System.Array.Copy(i_memoryFile.GetBytes(), 0, temp, 0, i_length);
				i_memoryFile.SetBytes(temp);
			}
			i_closed = true;
			return true;
		}

		public override void Copy(int oldAddress, int oldAddressOffset, int newAddress, int
			 newAddressOffset, int length)
		{
			byte[] bytes = MemoryFileBytes(newAddress + newAddressOffset + length);
			System.Array.Copy(bytes, oldAddress + oldAddressOffset, bytes, newAddress + newAddressOffset
				, length);
		}

		internal override void EmergencyClose()
		{
			base.EmergencyClose();
			i_closed = true;
		}

		internal override long FileLength()
		{
			return i_length;
		}

		internal override string FileName()
		{
			return "Memory File";
		}

		internal override bool HasShutDownHook()
		{
			return false;
		}

		internal override bool NeedsLockFileThread()
		{
			return false;
		}

		private void Open()
		{
			byte[] bytes = i_memoryFile.GetBytes();
			if (bytes == null || bytes.Length == 0)
			{
				i_memoryFile.SetBytes(new byte[i_memoryFile.GetInitialSize()]);
				ConfigureNewFile();
				Write(false);
				WriteHeader(false);
			}
			else
			{
				i_length = bytes.Length;
				ReadThis();
			}
		}

		internal override void ReadBytes(byte[] a_bytes, int a_address, int a_length)
		{
			try
			{
				System.Array.Copy(i_memoryFile.GetBytes(), a_address, a_bytes, 0, a_length);
			}
			catch (System.Exception e)
			{
				com.db4o.inside.Exceptions4.ThrowRuntimeException(13, e);
			}
		}

		internal override void ReadBytes(byte[] bytes, int address, int addressOffset, int
			 length)
		{
			ReadBytes(bytes, address + addressOffset, length);
		}

		public override void SyncFiles()
		{
		}

		internal override bool WriteAccessTime()
		{
			return true;
		}

		internal override void WriteBytes(com.db4o.YapReader a_bytes, int address, int addressOffset
			)
		{
			int fullAddress = address + addressOffset;
			int length = a_bytes.GetLength();
			System.Array.Copy(a_bytes._buffer, 0, MemoryFileBytes(fullAddress + length), fullAddress
				, length);
		}

		private byte[] MemoryFileBytes(int a_lastByte)
		{
			byte[] bytes = i_memoryFile.GetBytes();
			if (a_lastByte > i_length)
			{
				if (a_lastByte > bytes.Length)
				{
					int increase = a_lastByte - bytes.Length;
					if (increase < i_memoryFile.GetIncrementSizeBy())
					{
						increase = i_memoryFile.GetIncrementSizeBy();
					}
					byte[] temp = new byte[bytes.Length + increase];
					System.Array.Copy(bytes, 0, temp, 0, bytes.Length);
					i_memoryFile.SetBytes(temp);
					bytes = temp;
				}
				i_length = a_lastByte;
			}
			return bytes;
		}

		public override void WriteXBytes(int a_address, int a_length)
		{
		}
	}
}
