/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o.inside.btree
{
	public abstract class BTreePatch
	{
		protected com.db4o.inside.btree.BTreePatch _previous;

		protected com.db4o.inside.btree.BTreePatch _next;

		protected readonly com.db4o.Transaction _transaction;

		internal readonly object _object;

		public BTreePatch(com.db4o.Transaction transaction, object obj)
		{
			_transaction = transaction;
			_object = obj;
		}

		public virtual com.db4o.inside.btree.BTreePatch Append(com.db4o.inside.btree.BTreePatch
			 patch)
		{
			if (_transaction == patch._transaction)
			{
				patch._next = _next;
				return patch;
			}
			if (_next == null)
			{
				_next = patch;
			}
			else
			{
				_next = _next.Append(patch);
			}
			return this;
		}

		public virtual object Commit(com.db4o.Transaction trans, com.db4o.inside.btree.BTree
			 btree)
		{
			return Commit(trans, btree, true);
		}

		private object Commit(com.db4o.Transaction trans, com.db4o.inside.btree.BTree btree
			, bool firstInList)
		{
			if (_transaction == trans)
			{
				if (_next != null)
				{
					return _next;
				}
				return Committed(btree);
			}
			if (_next != null)
			{
				object newNext = _next.Commit(trans, btree, false);
				if (newNext is com.db4o.inside.btree.BTreePatch)
				{
					_next = (com.db4o.inside.btree.BTreePatch)newNext;
				}
				else
				{
					_next = null;
				}
			}
			return this;
		}

		protected abstract object Committed(com.db4o.inside.btree.BTree btree);

		public virtual com.db4o.inside.btree.BTreePatch ForTransaction(com.db4o.Transaction
			 trans)
		{
			if (_transaction == trans)
			{
				return this;
			}
			if (_next == null)
			{
				return null;
			}
			return _next.ForTransaction(trans);
		}

		public virtual object GetObject(com.db4o.Transaction trans)
		{
			com.db4o.inside.btree.BTreePatch patch = ForTransaction(trans);
			if (patch == null)
			{
				return com.db4o.foundation.No4.INSTANCE;
			}
			return patch.GetObject();
		}

		protected abstract object GetObject();

		public virtual object Rollback(com.db4o.Transaction trans, com.db4o.inside.btree.BTree
			 btree)
		{
			return Rollback(trans, btree, true);
		}

		public virtual object Rollback(com.db4o.Transaction trans, com.db4o.inside.btree.BTree
			 btree, bool firstInList)
		{
			if (_transaction == trans)
			{
				if (_next != null)
				{
					return _next;
				}
				return RolledBack(btree);
			}
			if (_next != null)
			{
				object newNext = _next.Rollback(trans, btree, false);
				if (newNext is com.db4o.inside.btree.BTreePatch)
				{
					_next = (com.db4o.inside.btree.BTreePatch)newNext;
				}
				else
				{
					_next = null;
				}
			}
			return this;
		}

		protected abstract object RolledBack(com.db4o.inside.btree.BTree btree);

		public override string ToString()
		{
			if (_object == null)
			{
				return "[NULL]";
			}
			return _object.ToString();
		}
	}
}
