/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o.inside.diagnostic
{
	/// <exclude></exclude>
	public class DiagnosticProcessor : com.db4o.diagnostic.DiagnosticConfiguration, com.db4o.foundation.DeepClone
	{
		private com.db4o.foundation.Collection4 _listeners;

		public DiagnosticProcessor()
		{
		}

		private DiagnosticProcessor(com.db4o.foundation.Collection4 listeners)
		{
			_listeners = listeners;
		}

		public virtual void AddListener(com.db4o.diagnostic.DiagnosticListener listener)
		{
			if (_listeners == null)
			{
				_listeners = new com.db4o.foundation.Collection4();
			}
			_listeners.Add(listener);
		}

		public virtual void CheckClassHasFields(com.db4o.YapClass yc)
		{
			com.db4o.YapField[] fields = yc.i_fields;
			if (fields != null && fields.Length == 0)
			{
				string name = yc.GetName();
				string[] ignoredPackages = new string[] { "java.util." };
				for (int i = 0; i < ignoredPackages.Length; i++)
				{
					if (name.IndexOf(ignoredPackages[i]) == 0)
					{
						return;
					}
				}
				if (IsDb4oClass(yc))
				{
					return;
				}
				OnDiagnostic(new com.db4o.diagnostic.ClassHasNoFields(name));
			}
		}

		public virtual void CheckUpdateDepth(int depth)
		{
			if (depth > 1)
			{
				OnDiagnostic(new com.db4o.diagnostic.UpdateDepthGreaterOne(depth));
			}
		}

		public virtual object DeepClone(object context)
		{
			return _listeners != null ? new com.db4o.inside.diagnostic.DiagnosticProcessor(new 
				com.db4o.foundation.Collection4(_listeners)) : new com.db4o.inside.diagnostic.DiagnosticProcessor
				();
		}

		public virtual bool Enabled()
		{
			return _listeners != null;
		}

		private bool IsDb4oClass(com.db4o.YapClass yc)
		{
			string name = yc.GetName();
			if (name.IndexOf("com.db4o.test") == 0)
			{
				return false;
			}
			return name.IndexOf("com.db4o.") == 0;
		}

		public virtual void LoadedFromClassIndex(com.db4o.YapClass yc)
		{
			if (IsDb4oClass(yc))
			{
				return;
			}
			OnDiagnostic(new com.db4o.diagnostic.LoadedFromClassIndex(yc.GetName()));
		}

		public virtual void NativeQueryUnoptimized(com.db4o.query.Predicate predicate)
		{
			OnDiagnostic(new com.db4o.diagnostic.NativeQueryNotOptimized(predicate));
		}

		private void OnDiagnostic(com.db4o.diagnostic.Diagnostic d)
		{
			if (_listeners == null)
			{
				return;
			}
			com.db4o.foundation.Iterator4 i = _listeners.Iterator();
			while (i.HasNext())
			{
				((com.db4o.diagnostic.DiagnosticListener)i.Next()).OnDiagnostic(d);
			}
		}

		public virtual void RemoveAllListeners()
		{
			_listeners = null;
		}
	}
}
