/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o.inside.query {

	using System;
	using System.Reflection;

	using com.db4o.nativequery.expr;

	/// <summary>
	/// This is the base class for tools capable
	/// too create an expression tree from a method.
	/// </summary>
	public abstract class ExpressionBuilder {

        protected static ICachingStrategy _assemblyCachingStrategy = new SingleItemCachingStrategy();

        protected static ICachingStrategy _expressionCachingStrategy = new SingleItemCachingStrategy();

		public static ICachingStrategy AssemblyCachingStrategy
		{
			get
			{
				return _assemblyCachingStrategy;
			}

			set
			{
				if (null == value) throw new ArgumentNullException("AssemblyCachingStrategy");
				_assemblyCachingStrategy = value;
			}
		}

        public static ICachingStrategy ExpressionCachingStrategy
        {
            get
            {
                return _expressionCachingStrategy;
            }

            set
            {
                if (null == value) throw new ArgumentNullException("ExpressionCachingStrategy");
                _expressionCachingStrategy = value;
            }
        }

		/// <summary>
		/// Create an expression tree from a method
		/// </summary>
		/// <param name="method">A method</param>
		/// <returns>An expression Tree</returns>
		public abstract Expression FromMethod (MethodBase method);
	}
}
