/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o.nativequery.optimization
{
	public class SODAQueryBuilder
	{
		private class SODAQueryVisitor : com.db4o.nativequery.expr.ExpressionVisitor
		{
			private object _predicate;

			private com.db4o.query.Query _query;

			private com.db4o.query.Constraint _constraint;

			internal SODAQueryVisitor(com.db4o.query.Query query, object predicate)
			{
				_query = query;
				_predicate = predicate;
			}

			public virtual void Visit(com.db4o.nativequery.expr.AndExpression expression)
			{
				expression.Left().Accept(this);
				com.db4o.query.Constraint left = _constraint;
				expression.Right().Accept(this);
				left.And(_constraint);
				_constraint = left;
			}

			public virtual void Visit(com.db4o.nativequery.expr.BoolConstExpression expression
				)
			{
			}

			public virtual void Visit(com.db4o.nativequery.expr.OrExpression expression)
			{
				expression.Left().Accept(this);
				com.db4o.query.Constraint left = _constraint;
				expression.Right().Accept(this);
				left.Or(_constraint);
				_constraint = left;
			}

			public virtual void Visit(com.db4o.nativequery.expr.ComparisonExpression expression
				)
			{
				com.db4o.query.Query subQuery = _query;
				com.db4o.foundation.Iterator4 fieldNameIterator = FieldNames(expression.Left());
				while (fieldNameIterator.HasNext())
				{
					subQuery = subQuery.Descend((string)fieldNameIterator.Next());
				}
				com.db4o.nativequery.optimization.ComparisonQueryGeneratingVisitor visitor = new 
					com.db4o.nativequery.optimization.ComparisonQueryGeneratingVisitor(_predicate);
				expression.Right().Accept(visitor);
				_constraint = subQuery.Constrain(visitor.Value());
				if (!expression.Op().Equals(com.db4o.nativequery.expr.cmp.ComparisonOperator.EQUALS
					))
				{
					if (expression.Op().Equals(com.db4o.nativequery.expr.cmp.ComparisonOperator.GREATER
						))
					{
						_constraint.Greater();
					}
					else
					{
						if (expression.Op().Equals(com.db4o.nativequery.expr.cmp.ComparisonOperator.SMALLER
							))
						{
							_constraint.Smaller();
						}
						else
						{
							if (expression.Op().Equals(com.db4o.nativequery.expr.cmp.ComparisonOperator.CONTAINS
								))
							{
								_constraint.Contains();
							}
							else
							{
								if (expression.Op().Equals(com.db4o.nativequery.expr.cmp.ComparisonOperator.STARTSWITH
									))
								{
									_constraint.StartsWith(true);
								}
								else
								{
									if (expression.Op().Equals(com.db4o.nativequery.expr.cmp.ComparisonOperator.ENDSWITH
										))
									{
										_constraint.EndsWith(true);
									}
									else
									{
										throw new j4o.lang.RuntimeException("Can't handle constraint: " + expression.Op()
											);
									}
								}
							}
						}
					}
				}
			}

			public virtual void Visit(com.db4o.nativequery.expr.NotExpression expression)
			{
				expression.Expr().Accept(this);
				_constraint.Not();
			}

			private com.db4o.foundation.Iterator4 FieldNames(com.db4o.nativequery.expr.cmp.FieldValue
				 fieldValue)
			{
				com.db4o.foundation.Collection4 coll = new com.db4o.foundation.Collection4();
				com.db4o.nativequery.expr.cmp.ComparisonOperand curOp = fieldValue;
				while (curOp is com.db4o.nativequery.expr.cmp.FieldValue)
				{
					com.db4o.nativequery.expr.cmp.FieldValue curField = (com.db4o.nativequery.expr.cmp.FieldValue
						)curOp;
					coll.Add(curField.FieldName());
					curOp = curField.Parent();
				}
				return coll.Iterator();
			}
		}

		public virtual void OptimizeQuery(com.db4o.nativequery.expr.Expression expr, com.db4o.query.Query
			 query, object predicate)
		{
			expr.Accept(new com.db4o.nativequery.optimization.SODAQueryBuilder.SODAQueryVisitor
				(query, predicate));
		}
	}
}
