/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace com.db4o.reflect.net
{
	/// <summary>Reflection implementation for Class to map to .NET reflection.</summary>
	/// <remarks>Reflection implementation for Class to map to .NET reflection.</remarks>
	public class NetClass : com.db4o.reflect.ReflectClass
	{
		private readonly com.db4o.reflect.Reflector _reflector;

		private readonly j4o.lang.Class _clazz;

		private readonly System.Type _type;

		private com.db4o.reflect.ReflectConstructor _constructor;

		private object[] constructorParams;

		public NetClass(com.db4o.reflect.Reflector reflector, j4o.lang.Class clazz)
		{
			_reflector = reflector;
			_clazz = clazz;
			_type = clazz.GetNetType();
		}

		public virtual com.db4o.reflect.ReflectClass GetComponentType()
		{
			return _reflector.ForClass(_clazz.GetComponentType());
		}

		public virtual com.db4o.reflect.ReflectConstructor[] GetDeclaredConstructors()
		{
			j4o.lang.reflect.Constructor[] constructors = _clazz.GetDeclaredConstructors();
			com.db4o.reflect.ReflectConstructor[] reflectors = new com.db4o.reflect.ReflectConstructor
				[constructors.Length];
			for (int i = 0; i < constructors.Length; i++)
			{
				reflectors[i] = new com.db4o.reflect.net.NetConstructor(_reflector, constructors[i]);
			}
			return reflectors;
		}

		public virtual com.db4o.reflect.ReflectField GetDeclaredField(string name)
		{
			try
			{
				return new com.db4o.reflect.net.NetField(_reflector, _clazz.GetDeclaredField(name));
			}
			catch (System.Exception e)
			{
				return null;
			}
		}

		public virtual com.db4o.reflect.ReflectField[] GetDeclaredFields()
		{
			j4o.lang.reflect.Field[] fields = _clazz.GetDeclaredFields();
			com.db4o.reflect.ReflectField[] reflectors = new com.db4o.reflect.ReflectField[fields.Length];
			for (int i = 0; i < reflectors.Length; i++)
			{
				reflectors[i] = new com.db4o.reflect.net.NetField(_reflector, fields[i]);
			}
			return reflectors;
		}

		public virtual com.db4o.reflect.ReflectClass GetDelegate()
		{
			return this;
		}

		public virtual com.db4o.reflect.ReflectMethod GetMethod(
			string methodName, 
			com.db4o.reflect.ReflectClass[] paramClasses)
		{
			try
			{
				j4o.lang.reflect.Method method = _clazz.GetMethod(methodName, com.db4o.reflect.net.NetReflector
					.ToNative(paramClasses));
				if (method == null)
				{
					return null;
				}
				return new com.db4o.reflect.net.NetMethod(_reflector, method);
			}
			catch (System.Exception e)
			{
				return null;
			}
		}

		public virtual string GetName()
		{
			return _clazz.GetName();
		}

		public virtual com.db4o.reflect.ReflectClass GetSuperclass()
		{
			return _reflector.ForClass(_clazz.GetSuperclass());
		}

		public virtual bool IsAbstract()
		{
			return j4o.lang.reflect.Modifier.IsAbstract(_clazz.GetModifiers());
		}

		public virtual bool IsArray()
		{
			return _clazz.IsArray();
		}

		public virtual bool IsAssignableFrom(com.db4o.reflect.ReflectClass type)
		{
			if (!(type is com.db4o.reflect.net.NetClass))
			{
				return false;
			}
			return _clazz.IsAssignableFrom(((com.db4o.reflect.net.NetClass)type).GetJavaClass(
				));
		}

		public virtual bool IsInstance(object obj)
		{
			return _clazz.IsInstance(obj);
		}

		public virtual bool IsInterface()
		{
			return _clazz.IsInterface();
		}

		public virtual bool IsCollection() 
		{
			return _reflector.IsCollection(this);
		}

		public virtual bool IsPrimitive()
		{
			return _clazz.IsPrimitive();
		}
		
		public virtual bool IsSecondClass() 
		{
			return IsPrimitive();
		}

		public virtual object NewInstance()
		{
			try
			{
				if (_constructor == null)
				{
					return _clazz.NewInstance();
				}
				return _constructor.NewInstance(constructorParams);
			}
			catch (System.Exception t)
			{
			}
			return null;
		}

		internal virtual j4o.lang.Class GetJavaClass()
		{
			return _clazz;
		}

		public virtual System.Type GetNetType() 
		{
			return _type;
		}

		public virtual com.db4o.reflect.Reflector Reflector()
		{
			return _reflector;
		}

		public virtual bool SkipConstructor(bool flag)
		{
#if !CF_1_0 && !CF_2_0
			if (flag) 
			{
				ReflectConstructor constructor = new SerializationConstructor(GetNetType());
				if (constructor != null)
				{
					try
					{
						object o = constructor.NewInstance(null);
						if (o != null)
						{
							UseConstructor(constructor, null);
							return true;
						}
					}
					catch (System.Exception e)
					{
					}
				}
			}
#endif
			UseConstructor(null, null);
			return false;
		}

		public override string ToString()
		{
			return "CClass: " + _clazz.GetName();
		}

		public virtual void UseConstructor(
			com.db4o.reflect.ReflectConstructor constructor, 
			object[] _params)
		{
			_constructor = constructor;
			constructorParams = _params;
		}
		
		public virtual object[] ToArray(object obj) 
		{
			// handled in GenericClass
			return null;
		}
	}
}
