/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Reflection;
using System.Collections;

namespace j4o.lang.reflect
{
    public class Field
    {
        private FieldInfo _fieldInfo;
        private EventInfo _eventInfo;
        private Class _fieldClass;
        private int _modifiers;

        private static IList _transientMarkers;

        public Field(FieldInfo fieldInfo, EventInfo eventInfo)
        {
            _fieldInfo = fieldInfo;
            _eventInfo = eventInfo;
        	_modifiers = -1;
        }
        
        public Object Get(Object obj)
        {
            return _fieldInfo.GetValue(obj);
        }

        public int GetModifiers()
        {
            if (_modifiers == -1)
            {
            	_modifiers = ComposeModifiers();
            }
            return _modifiers;
        }

        private int ComposeModifiers()
        {
            int modifiers = 0;
            if (_fieldInfo.IsFamilyAndAssembly)
            {
                modifiers |= Modifier.PROTECTED;
            }
            if (_fieldInfo.IsInitOnly)
            {
                modifiers |= Modifier.FINAL;
            }
            if (_fieldInfo.IsLiteral)
            {
                modifiers |= Modifier.FINAL;
                modifiers |= Modifier.STATIC;
            }
            if (_fieldInfo.IsPrivate)
            {
                modifiers |= Modifier.PRIVATE;
            }
            if (_fieldInfo.IsPublic)
            {
                modifiers |= Modifier.PUBLIC;
            }
            if (_fieldInfo.IsStatic)
            {
                modifiers |= Modifier.STATIC;
            }

            if (CheckForTransient(_fieldInfo.GetCustomAttributes(true)))
            {
                modifiers |= Modifier.TRANSIENT;
            }
            else if (_eventInfo != null)
            {
                if (CheckForTransient(_eventInfo.GetCustomAttributes(true)))
                {
                    modifiers |= Modifier.TRANSIENT;
                }
            }
            return modifiers;
        }

		private bool CheckForTransient(object[] attributes)
		{
			if (attributes == null) return false;

			foreach (object attribute in attributes)
			{
				string attributeName = attribute.ToString();
				if ("com.db4o.Transient" == attributeName) return true;
				if (_transientMarkers == null) continue;
				if (_transientMarkers.Contains(attributeName)) return true;
			}
			return false;
		}

        public String GetName()
        {
            return _fieldInfo.Name;
        }

        public Class GetFieldType()
        {
            if (_fieldClass == null)
            {
            	_fieldClass = Class.GetClassForType(_fieldInfo.FieldType);
            }
            return _fieldClass;
        }

        public static void MarkTransient(string attributeName)
        {
            if (_transientMarkers == null)
            {
                _transientMarkers = new ArrayList();
            }
            else if (_transientMarkers.Contains(attributeName))
            {
                return;
            }
            _transientMarkers.Add(attributeName);
        }

        public void Set(Object obj, Object value)
        {
        	_fieldInfo.SetValue(obj, value);
        }
    }
}
