/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Reflection;
using com.db4o.query;
using j4o.lang;
using j4o.lang.reflect;

namespace com.db4o.test.cs
{
	/// <summary>
	/// testing deactivation and zero date.
	/// </summary>
	public class CsDate
	{
        DateTime dateTime;

		bool flag = true;

		public CsDate()
		{
		}

		public CsDate(DateTime value)
		{
			dateTime = value;
		}

        public void Store()
		{
            Tester.DeleteAllInstances(this);
            Tester.Store(new CsDate(DateTime.Now));
        }

		public void TestTrivialQuery() 
		{
			Query q = Tester.Query();
			q.Constrain(typeof(CsDate));
			ObjectSet os = q.Execute();
			Tester.Ensure(os.Size() == 1);
		}

		public void TestQueryByExample() 
		{
			CsDate template = new CsDate();
			Tester.GetOne(template);

			template.dateTime = new DateTime(0);
			Tester.GetOne(template);

			template.dateTime = new DateTime(100);
			ObjectSet os = Tester.ObjectContainer().Get(template);
			Tester.Ensure(os.Size() == 0);
		}

		public void TestDeactivation()
		{
			CsDate template = new CsDate(new DateTime(100));
            Tester.ObjectContainer().Deactivate(template, int.MaxValue);
            Tester.Ensure(template.dateTime.Equals(new DateTime(0)));
        }

		public void TestSODA() 
		{
			DateTime before = DateTime.Now.AddDays(-1);
			DateTime after = DateTime.Now.AddDays(1);

			Query q = Tester.Query();
			q.Constrain(typeof(CsDate));
			q.Descend("dateTime").Constrain(before).Smaller();
			Tester.Ensure(0 == q.Execute().Size());

			q = Tester.Query();
			q.Constrain(typeof(CsDate));
			q.Descend("dateTime").Constrain(after).Greater();
			Tester.Ensure(0 == q.Execute().Size());

			q = Tester.Query();
			q.Constrain(typeof(CsDate));
			q.Descend("dateTime").Constrain(before).Greater();
			Tester.Ensure(1 == q.Execute().Size());

			q = Tester.Query();
			q.Constrain(typeof(CsDate));
			q.Descend("dateTime").Constrain(after).Smaller();
			Tester.Ensure(1 == q.Execute().Size());

			q = Tester.Query();
			q.Constrain(typeof(CsDate));
			q.Descend("flag").Constrain(true);
			q.Descend("dateTime").Constrain(before).Greater();
			q.Descend("dateTime").Constrain(after).Smaller();
			Tester.Ensure(1 == q.Execute().Size());

			q = Tester.Query();
			q.Constrain(typeof(CsDate));
			q.Descend("flag").Constrain(false);
			q.Descend("dateTime").Constrain(before).Greater();
			q.Descend("dateTime").Constrain(after).Smaller();
			Tester.Ensure(0 == q.Execute().Size());
		}
	}
}
