/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using System.Reflection;
using System.Text;

using Db4oTools.NativeQueries;

using com.db4o.inside.query;
using com.db4o.nativequery.expr;
using com.db4o.nativequery.optimization;
using com.db4o.query;

namespace com.db4o.test.nativequeries
{
	public class AbstractNativeQueriesTestCase
	{
		protected void AssertNQResult(object predicate, params object[] expected)
		{
			ObjectSet os = QueryFromPredicate(predicate).Execute();
			string actualString = ToString(os);
			if (!Tester.Ensure("Expected: " + ToString(expected) + ", Actual: " + actualString, expected.Length == os.Size()))
			{
				return;
			}

			foreach (object item in expected)
			{
				Tester.Ensure("Expected item: " + item + " but got: " + actualString, os.Contains(item));
			}
		}

		private string ToString(IEnumerable os)
		{	
			StringBuilder buffer = new StringBuilder();
			buffer.Append('[');
			foreach (object item in os)
			{
				if (buffer.Length > 1) buffer.Append(", ");
				buffer.Append(item.ToString());
			}
			buffer.Append(']');
			return buffer.ToString();
		}

		private Query QueryFromPredicate(object predicate)
		{
			MethodInfo match = predicate.GetType().GetMethod("Match");
			Expression expression = (new QueryExpressionBuilder ()).FromMethod(match);
			Query q = Tester.Query();
			q.Constrain(match.GetParameters()[0].ParameterType);
			new SODAQueryBuilder().OptimizeQuery(expression, q, predicate);
			return q;
		}
	}
}