/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using j4o.lang;
using com.db4o;
using com.db4o.query;
using com.db4o.test.soda.classes.simple;
using com.db4o.test.soda.classes.wrapper.untyped;
using com.db4o.test.soda.collections;
using com.db4o.test.soda.engines.db4o;
namespace com.db4o.test.soda {

    public class SodaTestThreadedRegression : SodaTest, Runnable {
        private static Object Lock = new Object();
        private static int RUNS = Platform4.IsCompact() ? 10 : 100;
        private STClass[] classes;
        private static volatile int runningThreads;
      
        public SodaTestThreadedRegression(STClass[] classes) : base() {
            this.classes = classes;
            SetSodaTestOn(classes);
        }
      
        public static void Main(String[] args) {
            testCases = 0;
            CascadeOnDelete(new STArrayListT());
            time = j4o.lang.JavaSystem.CurrentTimeMillis();
            engine = new STDb4o();
            engine.Reset();
            engine.Open();
			try {
				StartThread(new STClass[]{new STString()});
				StartThread(new STClass[]{new STInteger()});
				StartThread(new STClass[]{new STByte()});
				StartThread(new STClass[]{new STShort()});
				StartThread(new STClass[]{new STBooleanWU()});
				StartThread(new STClass[]{new STArrayListT()});
				do {
					try {
						Thread.Sleep(300);
					}  catch (Exception e) {
					}
				}while (runningThreads > 0);
        	} finally {
				engine.Close();
			}
		}
      
        private static void StartThread(STClass[] classes) {
            for (int i1 = 0; i1 < classes.Length; i1++) {
                if (!JdkOK(classes[i1])) {
                    JavaSystem._out.Println("Tester case can\'t run on this JDK: " + j4o.lang.Class.GetClassForObject(classes[i1]).GetName());
                    return;
                }
            }
            new Thread(new SodaTestThreadedRegression(classes)).Start();
        }
      
        protected override String Name() {
            return "S.O.D.A. threaded test";
        }
      
        public void Run() {
            String name1;
            lock (Lock) {
                runningThreads++;
                name1 = "R " + runningThreads + " ";
            }
            Thread.CurrentThread().SetName(name1);
            for (int i1 = 0; i1 < RUNS; i1++) {
                if (!QUIET) {
                    JavaSystem._out.Println(name1 + i1);
                }
                Store(classes);
                engine.Commit();
                Test(classes);
                for (int j1 = 0; j1 < classes.Length; j1++) {
                    Query q1 = engine.Query();
                    q1.Constrain(j4o.lang.Class.GetClassForObject(classes[j1]));
                    ObjectSet os = q1.Execute();
                    while (os.HasNext()) {
                        engine.Delete(os.Next());
                    }
                }
            }
            lock (Lock) {
                runningThreads--;
                if (runningThreads < 1) {
                    engine.Close();
                    Completed();
                }
            }
        }
      
        public static void CascadeOnDelete(Object obj) {
            Db4o.Configure().ObjectClass(j4o.lang.Class.GetClassForObject(obj).GetName()).CascadeOnDelete(true);
        }
    }
}
