/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using j4o.lang;
using j4o.util;
using com.db4o;
using com.db4o.query;
using com.db4o.test.soda;
namespace com.db4o.test.soda.collections {

    public class STHashtableU : STClass {
        [Transient] public static SodaTest st;
        internal Object col;
      
        public STHashtableU() : base() {
        }
      
        public STHashtableU(Object[] arr) : base() {
            System.Collections.Hashtable ht1 = new System.Collections.Hashtable();
            for (int i1 = 0; i1 < arr.Length; i1++) {
                ht1.Add(arr[i1], System.Convert.ToInt32(i1));
            }
            col = ht1;
        }
      
        public Object[] Store() {
            return new Object[]{
                                   new STHashtableU(),
                                   new STHashtableU(new Object[0]),
                                   new STHashtableU(new Object[]{
                                                                    System.Convert.ToInt32(0)            }),
                                   new STHashtableU(new Object[]{
                                                                    System.Convert.ToInt32(1),
                                                                    System.Convert.ToInt32(17),
                                                                    System.Convert.ToInt32(Int32.MaxValue - 1)            }),
                                   new STHashtableU(new Object[]{
                                                                    System.Convert.ToInt32(3),
                                                                    System.Convert.ToInt32(17),
                                                                    System.Convert.ToInt32(25),
                                                                    System.Convert.ToInt32(Int32.MaxValue - 2)            }),
                                   new STHashtableU(new Object[]{
                                                                    "foo",
                                                                    new STElement("bar", "barbar")            }),
                                   new STHashtableU(new Object[]{
                                                                    "foo2",
                                                                    new STElement("bar", "barbar2")            })         };
        }
      
        public void TestDefaultContainsInteger() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(new STHashtableU(new Object[]{
                                                          System.Convert.ToInt32(17)         }));
            st.Expect(q1, new Object[]{
                                          r1[3],
                                          r1[4]         });
        }
      
        public void TestDefaultContainsString() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(new STHashtableU(new Object[]{
                                                          "foo"         }));
            st.Expect(q1, new Object[]{
                                          r1[5]         });
        }
      
        public void TestDefaultContainsTwo() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(new STHashtableU(new Object[]{
                                                          System.Convert.ToInt32(17),
                                                          System.Convert.ToInt32(25)         }));
            st.Expect(q1, new Object[]{
                                          r1[4]         });
        }
      
        public void TestDescendOne() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(Class.GetClassForType(typeof(STHashtableU)));
            q1.Descend("col").Constrain(System.Convert.ToInt32(17));
            st.Expect(q1, new Object[]{
                                          r1[3],
                                          r1[4]         });
        }
      
        public void TestDescendTwo() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(Class.GetClassForType(typeof(STHashtableU)));
            Query qElements1 = q1.Descend("col");
            qElements1.Constrain(System.Convert.ToInt32(17));
            qElements1.Constrain(System.Convert.ToInt32(25));
            st.Expect(q1, new Object[]{
                                          r1[4]         });
        }
      
        public void TestDescendSmaller() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(Class.GetClassForType(typeof(STHashtableU)));
            Query qElements1 = q1.Descend("col");
            qElements1.Constrain(System.Convert.ToInt32(3)).Smaller();
            st.Expect(q1, new Object[]{
                                          r1[2],
                                          r1[3]         });
        }
      
        public void TestDefaultContainsObject() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(new STHashtableU(new Object[]{
                                                          new STElement("bar", null)         }));
            st.Expect(q1, new Object[]{
                                          r1[5],
                                          r1[6]         });
        }
      
        public void TestDescendToObject() {
            Query q1 = st.Query();
            Object[] r1 = Store();
            q1.Constrain(new STHashtableU());
            q1.Descend("col").Descend("foo1").Constrain("bar");
            st.Expect(q1, new Object[]{
                                          r1[5],
                                          r1[6]         });
        }
    }
}