/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using System.IO;
using Cecil.FlowAnalysis.CecilUtilities;
using Cecil.FlowAnalysis.ControlFlow;
using Mono.Cecil;
using Mono.Cecil.Cil;
using NUnit.Framework;

namespace Cecil.FlowAnalysis.Tests {
	public class AbstractControlFlowTestFixture : AbstractFlowAnalysisTestFixture {
		protected void RunTestCase (string name)
		{
			IMethodDefinition method = LoadTestCaseMethod (name);
			IControlFlowGraph cfg = FlowGraphFactory.CreateControlFlowGraph (method);
			Assert.AreEqual (normalize (LoadExpectedControlFlowString (name)), normalize (ToString (cfg)));
		}

		public static string ToString (IControlFlowGraph cfg)
		{
			StringWriter writer = new StringWriter ();
			FormatControlFlowGraph (writer, cfg);
			return writer.ToString ();
		}

		public static void FormatControlFlowGraph (TextWriter writer, IControlFlowGraph cfg)
		{
			int id = 1;
			foreach (IInstructionBlock block in cfg.Blocks) {
				writer.WriteLine ("block {0}:", id);
				writer.WriteLine ("\tbody:");
				foreach (IInstruction instruction in block) {
					writer.Write ("\t\t");
					CecilFormatter.WriteInstruction (writer, instruction);
					writer.WriteLine ();
				}
				IInstructionBlock[] successors = block.Successors;
				if (successors.Length > 0) {
					writer.WriteLine ("\tsuccessors:");
					foreach (IInstructionBlock successor in successors) {
						writer.WriteLine ("\t\tblock {0}", GetBlockId (cfg, successor));
					}
				}

				++id;
			}
		}

		private static int GetBlockId (IControlFlowGraph cfg, IInstructionBlock block)
		{
			return ((IList)cfg.Blocks).IndexOf (block) + 1;
		}

		private string LoadExpectedControlFlowString (string name)
		{
			return LoadTestCaseFile (name + "-cfg.txt");
		}
	}
}
