/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Cecil.FlowAnalysis.CodeStructure {
	
	public class AbstractCodeStructureVisitor : ICodeStructureVisitor {
		
		public virtual void Visit (ICodeElement node)
		{	
			if (null == node) return;
			node.Accept (this);
		}
		
		public virtual void Visit (System.Collections.ICollection collection)
		{	
			foreach (ICodeElement node in collection)
			{
				Visit (node);
			}
		}

		public virtual void Visit (IMethodInvocationExpression node)
		{
			Visit (node.Target);
			Visit (node.Arguments);
		}

		public virtual void Visit (IMethodReferenceExpression node)
		{
			Visit (node.Target);
		}

		public virtual void Visit (ILiteralExpression node)
		{
		}

		public virtual void Visit (IUnaryExpression node)
		{
			Visit (node.Operand);
		}

		public virtual void Visit (IBinaryExpression node)
		{
			Visit (node.Left);
			Visit (node.Right);
		}

		public virtual void Visit (IAssignExpression node)
		{
			Visit (node.Target);
			Visit (node.Expression);
		}

		public virtual void Visit (IArgumentReferenceExpression node)
		{
		}

		public virtual void Visit (IVariableReferenceExpression node)
		{
		}

		public virtual void Visit (IThisReferenceExpression node)
		{
		}

		public virtual void Visit (IFieldReferenceExpression node)
		{
			Visit (node.Target);
		}

		public virtual void Visit (IPropertyReferenceExpression node)
		{
			Visit (node.Target);
		}

		public virtual void Visit (IBlockStatement node)
		{
			Visit (node.Statements);
		}

		public virtual void Visit (IReturnStatement node)
		{
			Visit (node.Expression);
		}
	}
}
