/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using Cecil.FlowAnalysis.ActionFlow;
using Mono.Cecil.Cil;

namespace Cecil.FlowAnalysis.Impl.ActionFlow
{
	/// <summary>
	/// </summary>
	internal abstract class AbstractActionBlock : IActionBlock
	{
		ActionBlockCollection _predecessors = new ActionBlockCollection();
		private IInstruction _sourceInstruction;

		public AbstractActionBlock(IInstruction sourceInstruction)
		{
			if (null == sourceInstruction) throw new ArgumentNullException("sourceInstruction");
			_sourceInstruction = sourceInstruction;
		}

		public IInstruction SourceInstruction
		{
			get { return _sourceInstruction; }
		}

		public virtual IActionBlockCollection Predecessors
		{
			get { return _predecessors; }
		}

		public abstract ActionType ActionType
		{
			get;
		}

		public abstract IActionBlock[] Successors
		{
			get;
		}

		protected void AddAsPredecessorOf(IActionBlock block)
		{
			((ActionBlockCollection)block.Predecessors).Add(this);
		}

		public void RemovePredecessor(IActionBlock block)
		{
			_predecessors.Remove(block);
		}

		public abstract void ReplaceSuccessor(IActionBlock existing, IActionBlock newBlock);
	}
}
