/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using Cecil.FlowAnalysis.ActionFlow;
using Cecil.FlowAnalysis.ControlFlow;

namespace Cecil.FlowAnalysis.Impl.ActionFlow {
	internal class ActionFlowGraph : IActionFlowGraph {
		private ActionBlockCollection _blocks;
		private IControlFlowGraph _cfg;

		public ActionFlowGraph (IControlFlowGraph cfg, ActionBlockCollection blocks)
		{
			if (null == cfg) throw new ArgumentNullException ("cfg");
			if (null == blocks) throw new ArgumentNullException ("blocks");

			_cfg = cfg;
			_blocks = blocks;
		}

		public IControlFlowGraph ControlFlowGraph {
			get { return _cfg; }
		}

		public IActionBlockCollection Blocks {
			get { return _blocks; }
		}

		public bool IsBranchTarget (IActionBlock block)
		{
			if (null == block) throw new ArgumentNullException ("block");
			foreach (IActionBlock p in block.Predecessors) {
				switch (p.ActionType) {
				case ActionType.Branch:
					IBranchActionBlock br = (IBranchActionBlock) p;
					if (br.Target == block) return true;
					break;

				case ActionType.ConditionalBranch:
					IConditionalBranchActionBlock cbr = (IConditionalBranchActionBlock) p;
					if (cbr.Then == block) return true;
					break;
				}
			}
			return false;
		}

		public void ReplaceAt (int index, IActionBlock block)
		{
			if (null == block) throw new ArgumentNullException ("block");

			IActionBlock existing = _blocks [index];
			foreach (AbstractActionBlock p in existing.Predecessors.ToArray ()) {
				p.ReplaceSuccessor (existing, block);
			}
			Remove (existing);
			_blocks.Insert (index, block);
		}

		private void Remove (IActionBlock block)
		{
			foreach (AbstractActionBlock s in block.Successors) {
				s.RemovePredecessor (block);
				if (0 == s.Predecessors.Count) {
					Remove (s);
				}
			}
			_blocks.Remove (block);
		}
	}
}
