/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using Cecil.FlowAnalysis.ActionFlow;
using Cecil.FlowAnalysis.CodeStructure;
using Mono.Cecil.Cil;

namespace Cecil.FlowAnalysis.Impl.ActionFlow {
	/// <summary>
	/// </summary>
	internal class ConditionalBranchActionBlock : AbstractActionBlock, IConditionalBranchActionBlock {
		IExpression _condition;
		private IActionBlock _then;
		private IActionBlock _else;

		public ConditionalBranchActionBlock (IInstruction sourceInstruction, IExpression condition)
		: base(sourceInstruction)
		{
			if (null == condition) throw new ArgumentNullException ("condition");
			_condition = condition;
		}

		public void SetTargets (IActionBlock then, IActionBlock else_)
		{
			if (null == then) throw new ArgumentNullException ("then");
			AddAsPredecessorOf (then);
			if (null != else_) AddAsPredecessorOf (else_);
			_then = then;
			_else = else_;
		}

		override public ActionType ActionType {
			get { return ActionType.ConditionalBranch; }
		}

		public IExpression Condition {
			get { return _condition; }
		}

		public IActionBlock Then {
			get { return _then; }
		}

		public IActionBlock Else {
			get { return _else; }
		}

		public override IActionBlock[] Successors {
			get {
				return _else != null
					? new IActionBlock [] { _then, _else }
					: new IActionBlock [] { _then };
			}
		}

		public override void ReplaceSuccessor (IActionBlock existing, IActionBlock newBlock)
		{
			if (existing == _then) {
				_then = newBlock;
			} else if (existing == _else) {
				_else = newBlock;
			} else {
				throw new ArgumentException ("existing");
			}
		}
	}
}
