/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using Cecil.FlowAnalysis.ControlFlow;
using Mono.Cecil.Cil;

namespace Cecil.FlowAnalysis.Impl.ControlFlow {
	internal class InstructionBlock : IInstructionBlock {
		public static readonly IInstructionBlock[] NoSuccessors = new IInstructionBlock[0];

		private IInstruction _firstInstruction;
		private IInstruction _lastInstruction;
		private IInstructionBlock[] _successors = NoSuccessors;

		internal InstructionBlock (IInstruction first)
		{
			if (null == first) throw new ArgumentNullException ("first");
			_firstInstruction = first;
		}

		internal void SetLastInstruction (IInstruction last)
		{
			if (null == last) throw new ArgumentNullException ("last");
			_lastInstruction = last;
		}

		internal void SetSuccessors (IInstructionBlock[] successors)
		{
			_successors = successors;
		}

		public IInstruction FirstInstruction {
			get {
				return _firstInstruction;
			}
		}

		public IInstruction LastInstruction {
			get {
				return _lastInstruction;
			}
		}

		public IInstructionBlock[] Successors {
			get {
				return _successors;
			}
		}

		public int CompareTo (object obj)
		{
			return _firstInstruction.Offset.CompareTo (((InstructionBlock)obj).FirstInstruction.Offset);
		}

		public IEnumerator GetEnumerator ()
		{
			ArrayList instructions = new ArrayList ();
			IInstruction instruction = _firstInstruction;
			while (true) {
				instructions.Add (instruction);
				if (instruction == _lastInstruction) break;
				instruction = instruction.Next;
			}
			return instructions.GetEnumerator ();
		}
	}
}
