/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using Mono.Cecil.Cil;

namespace Cecil.FlowAnalysis.CecilUtilities {
	
	public class InstructionDispatcher {
		
		public static void Dispatch (IInstruction instruction, IInstructionVisitor visitor)
		{
			InstructionVisitorDelegate handler = (InstructionVisitorDelegate)_handlers[instruction.OpCode.Value];
			if (null == handler) throw new ArgumentException (CecilFormatter.FormatInstruction (instruction), "instruction");
			handler (visitor, instruction);
		}

		delegate void InstructionVisitorDelegate (IInstructionVisitor visitor, IInstruction instruction);

		static IDictionary _handlers = new Hashtable ();

		static InstructionDispatcher ()
		{
<%
	for instr in Instructions:
		opcodes = join("OpCodes.${code}" for code in instr.OpCodes, ", ")
%>			Bind (new InstructionVisitorDelegate (Dispatch${instr.OpCodes[0]}), ${opcodes});
<%
	end
%>		}

		static void Bind (InstructionVisitorDelegate handler, params OpCode[] opcodes)
		{
			foreach (OpCode op in opcodes)
			{
				_handlers.Add (op.Value, handler);
			}
		}
<%		
	for instr in Instructions:
%>
		static void Dispatch${instr.OpCodes[0]} (IInstructionVisitor visitor, IInstruction instruction)
		{
			visitor.On${instr.OpCodes[0]} (instruction);
		}
<%
	end
%>	}
}

