/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Binary {

	using Mono.Cecil.Metadata;

	public struct DataDirectory {

		public static readonly DataDirectory Zero = new DataDirectory(RVA.Zero, 0);

		RVA m_virtualAddress;
		uint m_size;

		public RVA VirtualAddress {
			get { return m_virtualAddress; }
			set { m_virtualAddress = value; }
		}

		public uint Size {
			get { return m_size; }
			set { m_size = value; }
		}

		public DataDirectory (RVA virtualAddress, uint size)
		{
			m_virtualAddress = virtualAddress;
			m_size = size;
		}

		public override int GetHashCode ()
		{
			return (m_virtualAddress.GetHashCode () ^ (int) m_size << 1);
		}

		public override bool Equals (object other)
		{
			if (other is DataDirectory) {
				DataDirectory odd = (DataDirectory)other;
				return (this.m_virtualAddress == odd.m_virtualAddress &&
				this.m_size == odd.m_size);
			}
			return false;
		}

		public override string ToString ()
		{
			return string.Format ("{0} [{1}]",
								  m_virtualAddress.ToString (), m_size.ToString ("X"));
		}

		public static bool operator == (DataDirectory one, DataDirectory other)
		{
			return one.Equals (other);
		}

		public static bool operator != (DataDirectory one, DataDirectory other)
		{
			return !one.Equals (other);
		}
	}
}
