/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Binary {

	public struct RVA {

		public static readonly RVA Zero = new RVA (0);

		uint m_rva;

		public uint Value {
			get { return m_rva; }
			set { m_rva = value; }
		}

		public RVA (uint rva)
		{
			m_rva = rva;
		}

		public override int GetHashCode ()
		{
			return (int) m_rva;
		}

		public override bool Equals (object other)
		{
			if (other is RVA)
				return this.m_rva == ((RVA) other).m_rva;

			return false;
		}

		public override string ToString ()
		{
			return string.Format ("0x{0}", m_rva.ToString ("X"));
		}

		public static bool operator == (RVA one, RVA other)
		{
			return one.Equals (other);
		}

		public static bool operator != (RVA one, RVA other)
		{
			return !one.Equals (other);
		}

		public static bool operator < (RVA one, RVA other)
		{
			return one.m_rva < other.m_rva;
		}

		public static bool operator > (RVA one, RVA other)
		{
			return one.m_rva > other.m_rva;
		}

		public static bool operator <= (RVA one, RVA other)
		{
			return one.m_rva <= other.m_rva;
		}

		public static bool operator >= (RVA one, RVA other)
		{
			return one.m_rva >= other.m_rva;
		}

		public static RVA operator + (RVA rva, uint x)
		{
			return new RVA (rva.m_rva + x);
		}

		public static RVA operator - (RVA rva, uint x)
		{
			return new RVA (rva.m_rva - x);
		}

		public static implicit operator RVA (uint val)
		{
			return val == 0 ? RVA.Zero : new RVA (val);
		}

		public static implicit operator uint (RVA rva)
		{
			return rva.m_rva;
		}
	}
}
