/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Cil {

	using System;
	using System.Collections;

	using Mono.Cecil.Cil;

	public sealed class ExceptionHandlerCollection : IExceptionHandlerCollection {

		IList m_items;
		MethodBody m_container;

		public event ExceptionHandlerEventHandler OnExceptionHandlerAdded;
		public event ExceptionHandlerEventHandler OnExceptionHandlerRemoved;

		public ExceptionHandler this [int index] {
			get { return m_items [index] as ExceptionHandler; }
			set { m_items [index] = value; }
		}

		object IIndexedCollection.this [int index] {
			get { return m_items [index]; }
		}

		public MethodBody Container {
			get { return m_container; }
		}

		public int Count {
			get { return m_items.Count; }
		}

		public bool IsSynchronized {
			get { return false; }
		}

		public object SyncRoot {
			get { return this; }
		}

		public ExceptionHandlerCollection (MethodBody container)
		{
			m_container = container;
			m_items = new ArrayList ();
		}

		public void Add (ExceptionHandler value)
		{
			if (OnExceptionHandlerAdded != null && !this.Contains (value))
				OnExceptionHandlerAdded (this, new ExceptionHandlerEventArgs (value));
			m_items.Add (value);
		}

		public void Clear ()
		{
			if (OnExceptionHandlerRemoved != null)
				foreach (ExceptionHandler item in this)
					OnExceptionHandlerRemoved (this, new ExceptionHandlerEventArgs (item));
			m_items.Clear ();
		}

		public bool Contains (ExceptionHandler value)
		{
			return m_items.Contains (value);
		}

		public int IndexOf (ExceptionHandler value)
		{
			return m_items.IndexOf (value);
		}

		public void Insert (int index, ExceptionHandler value)
		{
			if (OnExceptionHandlerAdded != null && !this.Contains (value))
				OnExceptionHandlerAdded (this, new ExceptionHandlerEventArgs (value));
			m_items.Insert (index, value);
		}

		public void Remove (ExceptionHandler value)
		{
			if (OnExceptionHandlerRemoved != null && this.Contains (value))
				OnExceptionHandlerRemoved (this, new ExceptionHandlerEventArgs (value));
			m_items.Remove (value);
		}

		public void RemoveAt (int index)
		{
			if (OnExceptionHandlerRemoved != null)
				OnExceptionHandlerRemoved (this, new ExceptionHandlerEventArgs (this [index]));
			m_items.RemoveAt (index);
		}

		public void CopyTo (Array ary, int index)
		{
			m_items.CopyTo (ary, index);
		}

		public IEnumerator GetEnumerator ()
		{
			return m_items.GetEnumerator ();
		}

		public void Accept (ICodeVisitor visitor)
		{
			visitor.VisitExceptionHandlerCollection (this);
		}
	}
}
