/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;
	using System.Collections;
	using System.Globalization;

	class CultureUtils {

		static IDictionary m_cultures;

		CultureUtils ()
		{
		}

		static void LoadCultures ()
		{
			if (m_cultures != null)
				return;

#if CF_1_0 || CF_2_0
			CultureInfo [] cultures = new CultureInfo[0];
#else
			CultureInfo [] cultures = CultureInfo.GetCultures (CultureTypes.AllCultures);
#endif
			m_cultures = new Hashtable (cultures.Length + 2);

			foreach (CultureInfo ci in cultures)
				if (!m_cultures.Contains (ci.Name))
					m_cultures.Add (ci.Name, ci);

			if (!m_cultures.Contains (string.Empty))
				m_cultures.Add (string.Empty, CultureInfo.InvariantCulture);

			m_cultures.Add ("neutral", CultureInfo.InvariantCulture);
		}

		public static bool IsValid (string culture)
		{
			if (culture == null)
				throw new ArgumentNullException ("culture");

			LoadCultures ();

			return m_cultures.Contains (culture);
		}

		public static CultureInfo GetCultureInfo (string culture)
		{
			if (IsValid (culture))
				return m_cultures [culture] as CultureInfo;

			return CultureInfo.InvariantCulture;
		}
	}
}
