/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;
	using System.Collections;

	public class GuidHeap : MetadataHeap {

		readonly IDictionary m_guids;

		public IDictionary Guids {
			get { return m_guids; }
		}

		public GuidHeap (MetadataStream stream) : base (stream, MetadataStream.GUID)
		{
			m_guids = new Hashtable ();
		}

		public Guid this [uint index] {
			get {
				if (index == 0)
					return new Guid (new byte [16]);

				int idx = (int) index - 1;

				if (m_guids.Contains (idx))
					return (Guid) m_guids [idx];

				if (idx + 16 > this.Data.Length)
					throw new IndexOutOfRangeException ();

				byte[] buffer = new byte [16];
				Buffer.BlockCopy (this.Data, idx, buffer, 0, 16);
				Guid res = new Guid (buffer);
				m_guids [idx] = res;
				return res;
			}
			set { m_guids [index] = value; }
		}

		public override void Accept (IMetadataVisitor visitor)
		{
			visitor.VisitGuidHeap (this);
		}
	}
}
