/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	public interface IMetadataVisitor {
		void VisitMetadataRoot (MetadataRoot root);
		void VisitMetadataRootHeader (MetadataRoot.MetadataRootHeader header);
		void VisitMetadataStreamCollection (MetadataStreamCollection streams);
		void VisitMetadataStream (MetadataStream stream);
		void VisitMetadataStreamHeader (MetadataStream.MetadataStreamHeader header);
		void VisitGuidHeap (GuidHeap heap);
		void VisitStringsHeap (StringsHeap heap);
		void VisitTablesHeap (TablesHeap heap);
		void VisitBlobHeap (BlobHeap heap);
		void VisitUserStringsHeap (UserStringsHeap heap);

		void TerminateMetadataRoot (MetadataRoot root);
	}

	public interface IMetadataTableVisitor {
		void VisitTableCollection (TableCollection coll);

		void VisitAssemblyTable (AssemblyTable table);
		void VisitAssemblyOSTable (AssemblyOSTable table);
		void VisitAssemblyProcessorTable (AssemblyProcessorTable table);
		void VisitAssemblyRefTable (AssemblyRefTable table);
		void VisitAssemblyRefOSTable (AssemblyRefOSTable table);
		void VisitAssemblyRefProcessorTable (AssemblyRefProcessorTable table);
		void VisitClassLayoutTable (ClassLayoutTable table);
		void VisitConstantTable (ConstantTable table);
		void VisitCustomAttributeTable (CustomAttributeTable table);
		void VisitDeclSecurityTable (DeclSecurityTable table);
		void VisitEventTable (EventTable table);
		void VisitEventMapTable (EventMapTable table);
		void VisitExportedTypeTable (ExportedTypeTable table);
		void VisitFieldTable (FieldTable table);
		void VisitFieldLayoutTable (FieldLayoutTable table);
		void VisitFieldMarshalTable (FieldMarshalTable table);
		void VisitFieldRVATable (FieldRVATable table);
		void VisitFileTable (FileTable table);
		void VisitGenericParamTable (GenericParamTable table);
		void VisitGenericParamConstraintTable (GenericParamConstraintTable table);
		void VisitImplMapTable (ImplMapTable table);
		void VisitInterfaceImplTable (InterfaceImplTable table);
		void VisitManifestResourceTable (ManifestResourceTable table);
		void VisitMemberRefTable (MemberRefTable table);
		void VisitMethodTable (MethodTable table);
		void VisitMethodImplTable (MethodImplTable table);
		void VisitMethodSemanticsTable (MethodSemanticsTable table);
		void VisitMethodSpecTable (MethodSpecTable table);
		void VisitModuleTable (ModuleTable table);
		void VisitModuleRefTable (ModuleRefTable table);
		void VisitNestedClassTable (NestedClassTable table);
		void VisitParamTable (ParamTable table);
		void VisitPropertyTable (PropertyTable table);
		void VisitPropertyMapTable (PropertyMapTable table);
		void VisitStandAloneSigTable (StandAloneSigTable table);
		void VisitTypeDefTable (TypeDefTable table);
		void VisitTypeRefTable (TypeRefTable table);
		void VisitTypeSpecTable (TypeSpecTable table);

		void TerminateTableCollection (TableCollection coll);
		IMetadataRowVisitor GetRowVisitor();
}

	public interface IMetadataRowVisitor {
		void VisitRowCollection (RowCollection coll);

		void VisitAssemblyRow (AssemblyRow row);
		void VisitAssemblyOSRow (AssemblyOSRow row);
		void VisitAssemblyProcessorRow (AssemblyProcessorRow row);
		void VisitAssemblyRefRow (AssemblyRefRow row);
		void VisitAssemblyRefOSRow (AssemblyRefOSRow row);
		void VisitAssemblyRefProcessorRow (AssemblyRefProcessorRow row);
		void VisitClassLayoutRow (ClassLayoutRow row);
		void VisitConstantRow (ConstantRow row);
		void VisitCustomAttributeRow (CustomAttributeRow row);
		void VisitDeclSecurityRow (DeclSecurityRow row);
		void VisitEventRow (EventRow row);
		void VisitEventMapRow (EventMapRow row);
		void VisitExportedTypeRow (ExportedTypeRow row);
		void VisitFieldRow (FieldRow row);
		void VisitFieldLayoutRow (FieldLayoutRow row);
		void VisitFieldMarshalRow (FieldMarshalRow row);
		void VisitFieldRVARow (FieldRVARow row);
		void VisitFileRow (FileRow row);
		void VisitGenericParamRow (GenericParamRow row);
		void VisitGenericParamConstraintRow (GenericParamConstraintRow row);
		void VisitImplMapRow (ImplMapRow row);
		void VisitInterfaceImplRow (InterfaceImplRow row);
		void VisitManifestResourceRow (ManifestResourceRow row);
		void VisitMemberRefRow (MemberRefRow row);
		void VisitMethodRow (MethodRow row);
		void VisitMethodImplRow (MethodImplRow row);
		void VisitMethodSemanticsRow (MethodSemanticsRow row);
		void VisitMethodSpecRow (MethodSpecRow row);
		void VisitModuleRow (ModuleRow row);
		void VisitModuleRefRow (ModuleRefRow row);
		void VisitNestedClassRow (NestedClassRow row);
		void VisitParamRow (ParamRow row);
		void VisitPropertyRow (PropertyRow row);
		void VisitPropertyMapRow (PropertyMapRow row);
		void VisitStandAloneSigRow (StandAloneSigRow row);
		void VisitTypeDefRow (TypeDefRow row);
		void VisitTypeRefRow (TypeRefRow row);
		void VisitTypeSpecRow (TypeSpecRow row);

		void TerminateRowCollection (RowCollection coll);
	}
}
