/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;

	using Mono.Cecil;

	public abstract class MetadataHeap : IMetadataVisitable  {

		MetadataStream m_stream;
		string m_name;
		byte [] m_data;

		public string Name {
			get { return m_name; }
		}

		public byte [] Data {
			get { return m_data; }
			set { m_data = value; }
		}

		public int IndexSize;

		internal MetadataHeap (MetadataStream stream, string name)
		{
			m_name = name;
			m_stream = stream;
		}

		public static MetadataHeap HeapFactory (MetadataStream stream)
		{
			switch (stream.Header.Name) {
			case MetadataStream.Tables :
			case MetadataStream.IncrementalTables :
				return new TablesHeap (stream);
			case MetadataStream.GUID :
				return new GuidHeap (stream);
			case MetadataStream.Strings :
				return new StringsHeap (stream);
			case MetadataStream.UserStrings :
				return new UserStringsHeap (stream);
			case MetadataStream.Blob :
				return new BlobHeap (stream);
			default :
				return null;
			}
		}

		public MetadataStream GetStream ()
		{
			return m_stream;
		}

		protected virtual byte [] ReadBytesFromStream (uint pos)
		{
			int start, length = Utilities.ReadCompressedInteger (m_data, (int) pos, out start);
			byte [] buffer = new byte [length];
			Buffer.BlockCopy (m_data, start, buffer, 0, length);
			return buffer;
		}

		public abstract void Accept (IMetadataVisitor visitor);
	}
}
