/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using Mono.Cecil.Binary;

	public sealed class MetadataRoot : IMetadataVisitable {

		MetadataRootHeader m_header;
		Image m_image;

		MetadataStreamCollection m_streams;

		public MetadataRootHeader Header {
			get { return m_header; }
			set { m_header = value; }
		}

		public MetadataStreamCollection Streams {
			get { return m_streams; }
			set { m_streams = value; }
		}

		internal MetadataRoot (Image img)
		{
			m_image = img;
		}

		public Image GetImage ()
		{
			return m_image;
		}

		public void Accept (IMetadataVisitor visitor)
		{
			visitor.VisitMetadataRoot (this);

			m_header.Accept (visitor);
			m_streams.Accept (visitor);

			visitor.TerminateMetadataRoot (this);
		}

		public sealed class MetadataRootHeader : IHeader, IMetadataVisitable {

			public const uint StandardSignature = 0x424a5342;

			public uint Signature;
			public ushort MinorVersion;
			public ushort MajorVersion;
			public uint Reserved;
			public string Version;
			public ushort Flags;
			public ushort Streams;

			internal MetadataRootHeader ()
			{
			}

			public void SetDefaultValues ()
			{
				Signature = StandardSignature;
				Reserved = 0;
				Flags = 0;
			}

			public void Accept (IMetadataVisitor visitor)
			{
				visitor.VisitMetadataRootHeader (this);
			}
		}
	}
}
