/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;
	using System.Collections;

	public class MetadataStreamCollection : ICollection, IMetadataVisitable {

		IList m_items;

		BlobHeap m_blobHeap;
		GuidHeap m_guidHeap;
		StringsHeap m_stringsHeap;
		UserStringsHeap m_usHeap;
		TablesHeap m_tablesHeap;

		public MetadataStream this [int index] {
			get { return m_items [index] as MetadataStream; }
			set { m_items [index] = value; }
		}

		public int Count {
			get { return m_items.Count; }
		}

		public bool IsSynchronized {
			get { return false; }
		}

		public object SyncRoot {
			get { return this; }
		}

		public BlobHeap BlobHeap {
			get {
				if (m_blobHeap == null)
					m_blobHeap = GetHeap (MetadataStream.Blob) as BlobHeap;
				return m_blobHeap;
			}
		}

		public GuidHeap GuidHeap {
			get {
				if (m_guidHeap == null)
					m_guidHeap = GetHeap (MetadataStream.GUID) as GuidHeap;
				return m_guidHeap;
			}
		}

		public StringsHeap StringsHeap {
			get {
				if (m_stringsHeap == null)
					m_stringsHeap = GetHeap (MetadataStream.Strings) as StringsHeap;
				return m_stringsHeap;
			}
		}

		public TablesHeap TablesHeap {
			get {
				if (m_tablesHeap == null)
					m_tablesHeap = GetHeap (MetadataStream.Tables) as TablesHeap;
				return m_tablesHeap;
			}
		}

		public UserStringsHeap UserStringsHeap {
			get {
				if (m_usHeap == null)
					m_usHeap = GetHeap (MetadataStream.UserStrings) as UserStringsHeap;
				return m_usHeap;
			}
		}

		public MetadataStreamCollection ()
		{
			m_items = new ArrayList (5);
		}

		private MetadataHeap GetHeap (string name)
		{
			MetadataHeap heap = null;
			for (int i = 0; i < m_items.Count; i++) {
				MetadataStream stream = m_items [i] as MetadataStream;
				if (stream.Heap.Name == name) {
					heap = stream.Heap;
					break;
				}
			}
			return heap;
		}

		internal void Add (MetadataStream value)
		{
			m_items.Add (value);
		}

		internal void Clear ()
		{
			m_items.Clear ();
		}

		public bool Contains (MetadataStream value)
		{
			return m_items.Contains (value);
		}

		public int IndexOf (MetadataStream value)
		{
			return m_items.IndexOf (value);
		}

		internal void Insert (int index, MetadataStream value)
		{
			m_items.Insert (index, value);
		}

		internal void Remove (MetadataStream value)
		{
			m_items.Remove (value);
		}

		internal void RemoveAt (int index)
		{
			m_items.Remove (index);
		}

		public void CopyTo (Array ary, int index)
		{
			m_items.CopyTo (ary, index);
		}

		public IEnumerator GetEnumerator ()
		{
			return m_items.GetEnumerator ();
		}

		public void Accept (IMetadataVisitor visitor)
		{
			visitor.VisitMetadataStreamCollection (this);

			for (int i = 0; i < m_items.Count; i++)
				this [i].Accept (visitor);
		}
	}
}
