/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	public struct MetadataToken {

		uint m_rid;
		TokenType m_type;

		public uint RID {
			get { return m_rid; }
		}

		public TokenType TokenType {
			get { return m_type; }
		}

		public static readonly MetadataToken Zero = new MetadataToken ((TokenType) 0, 0);

		public MetadataToken (TokenType table, uint rid)
		{
			m_type = table;
			m_rid = rid;
		}

		internal static MetadataToken FromMetadataRow (TokenType table, int rowIndex)
		{
			return new MetadataToken (table, (uint) rowIndex + 1);
		}

		public uint ToUInt ()
		{
			return (uint) m_type | m_rid;
		}

		public override int GetHashCode ()
		{
			return (int) ToUInt ();
		}

		public override bool Equals (object other)
		{
			if (other is MetadataToken) {
				MetadataToken o = (MetadataToken) other;
				return o.m_rid == m_rid && o.m_type == m_type;
			}

			return false;
		}

		public static bool operator == (MetadataToken one, MetadataToken other)
		{
			return one.Equals (other);
		}

		public static bool operator != (MetadataToken one, MetadataToken other)
		{
			return !one.Equals (other);
		}

		public override string ToString ()
		{
			return string.Format ("{0} [0x{1}]",
				m_type, m_rid.ToString ("x4"));
		}
	}
}
