/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;
	using System.Collections;

	public class RowCollection : ICollection, IMetadataRowVisitable {

		ArrayList m_items;

		public IMetadataRow this [int index] {
			get { return m_items [index] as IMetadataRow; }
			set { m_items [index] = value; }
		}

		public int Count {
			get { return m_items.Count; }
		}

		public bool IsSynchronized {
			get { return false; }
		}

		public object SyncRoot {
			get { return this; }
		}

		internal RowCollection (int size)
		{
			m_items = new ArrayList (size);
		}

		internal RowCollection ()
		{
			m_items = new ArrayList ();
		}

		internal void Add (IMetadataRow value)
		{
			m_items.Add (value);
		}

		public void Clear ()
		{
			m_items.Clear ();
		}

		public bool Contains (IMetadataRow value)
		{
			return m_items.Contains (value);
		}

		public int IndexOf (IMetadataRow value)
		{
			return m_items.IndexOf (value);
		}

		public void Insert (int index, IMetadataRow value)
		{
			m_items.Insert (index, value);
		}

		public void Remove (IMetadataRow value)
		{
			m_items.Remove (value);
		}

		public void RemoveAt (int index)
		{
			m_items.Remove (index);
		}

		public void CopyTo (Array ary, int index)
		{
			m_items.CopyTo (ary, index);
		}

		public void Sort (IComparer comp)
		{
			m_items.Sort (comp);
		}

		public IEnumerator GetEnumerator ()
		{
			return m_items.GetEnumerator ();
		}

		public void Accept (IMetadataRowVisitor visitor)
		{
			visitor.VisitRowCollection (this);

			for (int i = 0; i < m_items.Count; i++)
				this [i].Accept (visitor);

			visitor.TerminateRowCollection (this);
		}
	}
}
