/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;
	using System.Collections;

	public class TableCollection : ICollection, IMetadataTableVisitable	{

		IMetadataTable [] m_tables = new IMetadataTable [TablesHeap.MaxTableCount];

		TablesHeap m_heap;

		public IMetadataTable this [int index] {
			get { return m_tables [index]; }
			set { m_tables [index] = value; }
		}

		public int Count {
			get {
				return GetList ().Count;
			}
		}

		public bool IsSynchronized {
			get { return false; }
		}

		public object SyncRoot {
			get { return this; }
		}

		public TablesHeap Heap {
			get { return m_heap; }
		}

		internal TableCollection (TablesHeap heap)
		{
			m_heap = heap;
		}

		internal void Add (IMetadataTable value)
		{
			m_tables [value.Id] = value;
		}

		public bool Contains (IMetadataTable value)
		{
			return m_tables [value.Id] != null;
		}

		internal void Remove (IMetadataTable value)
		{
			m_tables [value.Id] = null;
		}

		public void CopyTo (Array array, int index)
		{
			GetList ().CopyTo (array, index);
		}

		internal IList GetList ()
		{
			IList tables = new ArrayList ();
			for (int i = 0; i < m_tables.Length; i++) {
				IMetadataTable table = m_tables [i];
				if (table != null)
					tables.Add (table);
			}

			return tables;
		}

		public IEnumerator GetEnumerator ()
		{
			return GetList ().GetEnumerator ();
		}

		public void Accept (IMetadataTableVisitor visitor)
		{
			visitor.VisitTableCollection (this);

			foreach (IMetadataTable table in GetList ())
				table.Accept (visitor);

			visitor.TerminateTableCollection (this);
		}
	}
}
