/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil.Metadata {

	using System;
	using System.Collections;
	using System.Text;

	public class UserStringsHeap : MetadataHeap {

		readonly IDictionary m_strings;

		public string this [uint offset] {
			get {
				string us = m_strings [offset] as string;
				if (us == null) {
					us = ReadStringAt ((int) offset);
					if (us != null && us.Length != 0)
						m_strings [offset] = us;
				}
				return us;
			}
			set { m_strings [offset] = value; }
		}

		internal UserStringsHeap (MetadataStream stream) : base(stream, MetadataStream.UserStrings)
		{
			m_strings = new Hashtable ();
		}

		string ReadStringAt (int offset)
		{
			int length = Utilities.ReadCompressedInteger (this.Data, offset, out offset);
			if (length == 0)
				return string.Empty;

			return Encoding.Unicode.GetString (this.Data, offset, length);
		}

		public override void Accept (IMetadataVisitor visitor)
		{
			visitor.VisitUserStringsHeap (this);
		}
	}
}
