/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil {

	using System;
	using System.Text;

	using Mono.Cecil.Signatures;

	public sealed class ArrayType : TypeSpecification, IArrayType {

		private ArrayDimensionCollection m_dimensions;

		public ArrayDimensionCollection Dimensions {
			get { return m_dimensions; }
		}

		public int Rank {
			get { return m_dimensions.Count; }
		}

		public bool IsSizedArray {
			get {
				if (this.Rank != 1)
					return false;
				IArrayDimension dim = m_dimensions [0];
				return dim.UpperBound == 0;
			}
		}

		public override string Name {
			get { return string.Concat (base.Name, Suffix ()); }
		}

		public override string FullName {
			get { return string.Concat (base.FullName, Suffix ()); }
		}

		string Suffix ()
		{
			StringBuilder sb = new StringBuilder ();
			sb.Append ("[");
			for (int i = 0; i < m_dimensions.Count; i++) {
				IArrayDimension dim = m_dimensions [i];
				string rank = dim.ToString ();
				if (i < m_dimensions.Count - 1)
					sb.Append (",");
				if (rank.Length > 0) {
					sb.Append (" ");
					sb.Append (rank);
				}
			}
			sb.Append ("]");
			return sb.ToString ();
		}

		internal ArrayType (TypeReference elementsType, ArrayShape shape) : base (elementsType)
		{
			m_dimensions = new ArrayDimensionCollection (this);
			for (int i = 0; i < shape.Rank; i++) {
				int lower = 0, upper = 0;
				if (i < shape.NumSizes)
					if (i < shape.NumLoBounds) {
						lower = shape.LoBounds [i];
						upper = shape.LoBounds [i] + shape.Sizes [i] - 1;
					} else
						upper = shape.Sizes [i] - 1;

				m_dimensions.Add (new ArrayDimension (lower, upper));
			}
		}

		public ArrayType (TypeReference elementsType) : base (elementsType)
		{
			m_dimensions = new ArrayDimensionCollection (this);
			m_dimensions.Add (new ArrayDimension (0, 0));
		}
	}
}
