/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil {

	using System;
	using System.Reflection;

	using Mono.Cecil.Metadata;

	public class AssemblyDefinition : IAssemblyDefinition {

		MetadataToken m_token;
		AssemblyNameDefinition m_asmName;
		ModuleDefinitionCollection m_modules;
		SecurityDeclarationCollection m_secDecls;
		CustomAttributeCollection m_customAttrs;
		MethodDefinition m_ep;
		TargetRuntime m_runtime;
		AssemblyKind m_kind;

		ModuleDefinition m_mainModule;
		StructureReader m_reader;

		public MetadataToken MetadataToken {
			get { return m_token; }
			set { m_token = value; }
		}

		public AssemblyNameDefinition Name {
			get { return m_asmName; }
		}

		public ModuleDefinitionCollection Modules {
			get { return m_modules; }
		}

		public SecurityDeclarationCollection SecurityDeclarations {
			get {
				if (m_secDecls == null)
					m_secDecls = new SecurityDeclarationCollection (this);

				return m_secDecls;
			}
		}

		public CustomAttributeCollection CustomAttributes {
			get {
				if (m_customAttrs == null)
					m_customAttrs = new CustomAttributeCollection (this);

				return m_customAttrs;
			}
		}

		public MethodDefinition EntryPoint {
			get { return m_ep; }
			set { m_ep = value as MethodDefinition; }
		}

		public TargetRuntime Runtime {
			get { return m_runtime; }
			set { m_runtime = value; }
		}

		public AssemblyKind Kind {
			get { return m_kind; }
			set { m_kind = value; }
		}

		public ModuleDefinition MainModule {
			get {
				if (m_mainModule == null)
					foreach (ModuleDefinition module in m_modules)
						if (module.Main)
							m_mainModule = module;

				return m_mainModule;
			}
		}

		internal StructureReader Reader {
			get { return m_reader; }
		}

		internal AssemblyDefinition (AssemblyNameDefinition name)
		{
			if (name == null)
				throw new ArgumentNullException ("name");

			m_asmName = name;
			m_modules = new ModuleDefinitionCollection (this);
		}

		internal AssemblyDefinition (AssemblyNameDefinition name, StructureReader reader) : this (name)
		{
			m_reader = reader;
		}

		public void Accept (IReflectionStructureVisitor visitor)
		{
			visitor.VisitAssemblyDefinition (this);

			m_asmName.Accept (visitor);
			m_modules.Accept (visitor);

			visitor.TerminateAssemblyDefinition (this);
		}

		public override string ToString ()
		{
			return m_asmName.FullName;
		}
	}
}
