/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil {

	using System;
	using System.Collections;

	using Mono.Cecil.Cil;

	public sealed class FieldDefinitionCollection : IFieldDefinitionCollection {

		IList m_items;
		TypeDefinition m_container;

		public event FieldDefinitionEventHandler OnFieldDefinitionAdded;
		public event FieldDefinitionEventHandler OnFieldDefinitionRemoved;

		public FieldDefinition this [int index] {
			get { return m_items [index] as FieldDefinition; }
			set { m_items [index] = value; }
		}

		object IIndexedCollection.this [int index] {
			get { return m_items [index]; }
		}

		public TypeDefinition Container {
			get { return m_container; }
		}

		public int Count {
			get { return m_items.Count; }
		}

		public bool IsSynchronized {
			get { return false; }
		}

		public object SyncRoot {
			get { return this; }
		}

		public FieldDefinitionCollection (TypeDefinition container)
		{
			m_container = container;
			m_items = new ArrayList ();
		}

		public void Add (FieldDefinition value)
		{
			if (OnFieldDefinitionAdded != null && !this.Contains (value))
				OnFieldDefinitionAdded (this, new FieldDefinitionEventArgs (value));
			m_items.Add (value);
		}

		public void Clear ()
		{
			if (OnFieldDefinitionRemoved != null)
				foreach (FieldDefinition item in this)
					OnFieldDefinitionRemoved (this, new FieldDefinitionEventArgs (item));
			m_items.Clear ();
		}

		public bool Contains (FieldDefinition value)
		{
			return m_items.Contains (value);
		}

		public int IndexOf (FieldDefinition value)
		{
			return m_items.IndexOf (value);
		}

		public void Insert (int index, FieldDefinition value)
		{
			if (OnFieldDefinitionAdded != null && !this.Contains (value))
				OnFieldDefinitionAdded (this, new FieldDefinitionEventArgs (value));
			m_items.Insert (index, value);
		}

		public void Remove (FieldDefinition value)
		{
			if (OnFieldDefinitionRemoved != null && this.Contains (value))
				OnFieldDefinitionRemoved (this, new FieldDefinitionEventArgs (value));
			m_items.Remove (value);
		}

		public void RemoveAt (int index)
		{
			if (OnFieldDefinitionRemoved != null)
				OnFieldDefinitionRemoved (this, new FieldDefinitionEventArgs (this [index]));
			m_items.RemoveAt (index);
		}

		public void CopyTo (Array ary, int index)
		{
			m_items.CopyTo (ary, index);
		}

		public IEnumerator GetEnumerator ()
		{
			return m_items.GetEnumerator ();
		}

		public FieldDefinition GetField (string name)
		{
			foreach (FieldDefinition field in this)
				if (field.Name == name)
					return field;

			return null;
		}

		public void Accept (IReflectionVisitor visitor)
		{
			visitor.VisitFieldDefinitionCollection (this);
		}
	}
}
