/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil {

	using System;
	using System.Collections;

	using Mono.Cecil.Cil;

	public sealed class MethodDefinitionCollection : IMethodDefinitionCollection {

		IList m_items;
		TypeDefinition m_container;

		public event MethodDefinitionEventHandler OnMethodDefinitionAdded;
		public event MethodDefinitionEventHandler OnMethodDefinitionRemoved;

		public MethodDefinition this [int index] {
			get { return m_items [index] as MethodDefinition; }
			set { m_items [index] = value; }
		}

		object IIndexedCollection.this [int index] {
			get { return m_items [index]; }
		}

		public TypeDefinition Container {
			get { return m_container; }
		}

		public int Count {
			get { return m_items.Count; }
		}

		public bool IsSynchronized {
			get { return false; }
		}

		public object SyncRoot {
			get { return this; }
		}

		public MethodDefinitionCollection (TypeDefinition container)
		{
			m_container = container;
			m_items = new ArrayList ();
		}

		public void Add (MethodDefinition value)
		{
			if (OnMethodDefinitionAdded != null && !this.Contains (value))
				OnMethodDefinitionAdded (this, new MethodDefinitionEventArgs (value));
			m_items.Add (value);
		}

		public void Clear ()
		{
			if (OnMethodDefinitionRemoved != null)
				foreach (MethodDefinition item in this)
					OnMethodDefinitionRemoved (this, new MethodDefinitionEventArgs (item));
			m_items.Clear ();
		}

		public bool Contains (MethodDefinition value)
		{
			return m_items.Contains (value);
		}

		public int IndexOf (MethodDefinition value)
		{
			return m_items.IndexOf (value);
		}

		public void Insert (int index, MethodDefinition value)
		{
			if (OnMethodDefinitionAdded != null && !this.Contains (value))
				OnMethodDefinitionAdded (this, new MethodDefinitionEventArgs (value));
			m_items.Insert (index, value);
		}

		public void Remove (MethodDefinition value)
		{
			if (OnMethodDefinitionRemoved != null && this.Contains (value))
				OnMethodDefinitionRemoved (this, new MethodDefinitionEventArgs (value));
			m_items.Remove (value);
		}

		public void RemoveAt (int index)
		{
			if (OnMethodDefinitionRemoved != null)
				OnMethodDefinitionRemoved (this, new MethodDefinitionEventArgs (this [index]));
			m_items.RemoveAt (index);
		}

		public void CopyTo (Array ary, int index)
		{
			m_items.CopyTo (ary, index);
		}

		public IEnumerator GetEnumerator ()
		{
			return m_items.GetEnumerator ();
		}

		public MethodDefinition [] GetMethod (string name)
		{
			ArrayList ret = new ArrayList ();
			foreach (MethodDefinition meth in this)
				if (meth.Name == name)
					ret.Add (meth);

			return ret.ToArray (typeof (MethodDefinition)) as MethodDefinition [];
		}

		public MethodDefinition GetMethod (string name, Type [] parameters)
		{
			foreach (MethodDefinition meth in this)
				if (meth.Name == name && meth.Parameters.Count == parameters.Length) {
					if (parameters.Length == 0)
						return meth;
					for (int i = 0; i < parameters.Length; i++)
						if (meth.Parameters [i].ParameterType.FullName == ReflectionHelper.GetTypeSignature (parameters [i]))
							return meth;
				}
			return null;
		}

		public MethodDefinition GetMethod (string name, ITypeReference [] parameters)
		{
			foreach (MethodDefinition meth in this)
				if (meth.Name == name && meth.Parameters.Count == parameters.Length) {
					if (parameters.Length == 0)
						return meth;
					for (int i = 0; i < parameters.Length; i++)
						if (meth.Parameters [i].ParameterType.FullName == parameters [i].FullName)
							return meth;
				}
			return null;
		}

		public MethodDefinition GetMethod (string name, ParameterDefinitionCollection parameters)
		{
			foreach (MethodDefinition meth in this)
				if (meth.Name == name && meth.Parameters.Count == parameters.Count) {
					if (parameters.Count == 0)
						return meth;
					for (int i = 0; i < parameters.Count; i++)
						if (meth.Parameters [i].ParameterType.FullName == parameters [i].ParameterType.FullName)
							return meth;	
				}
			return null;
		}

		public void Accept (IReflectionVisitor visitor)
		{
			visitor.VisitMethodDefinitionCollection (this);
		}
	}
}
