/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
namespace Mono.Cecil {

	using System;
	using System.Text;
	using System.Reflection;

	using Mono.Cecil.Binary;
	using Mono.Cecil.Cil;
	using Mono.Cecil.Signatures;

	public class MethodReference : MemberReference, IMethodReference {

		ParameterDefinitionCollection m_parameters;
		MethodReturnType m_returnType;

		bool m_hasThis;
		bool m_explicitThis;
		MethodCallingConvention m_callConv;
		GenericParameterCollection m_genparams;

		public virtual bool HasThis {
			get { return m_hasThis; }
			set { m_hasThis = value; }
		}

		public virtual bool ExplicitThis {
			get { return m_explicitThis; }
			set { m_explicitThis = value; }
		}

		public virtual MethodCallingConvention CallingConvention {
			get { return m_callConv; }
			set { m_callConv = value; }
		}

		public virtual ParameterDefinitionCollection Parameters {
			get {
				if (m_parameters == null)
					m_parameters = new ParameterDefinitionCollection (this);
				return m_parameters;
			}
		}

		public GenericParameterCollection GenericParameters {
			get {
				if (m_genparams == null)
					m_genparams = new GenericParameterCollection (this);
				return m_genparams;
			}
		}

		public virtual MethodReturnType ReturnType {
			get { return m_returnType;}
			set { m_returnType = value; }
		}

		internal MethodReference (string name, bool hasThis,
			bool explicitThis, MethodCallingConvention callConv) : this (name)
		{
			m_parameters = new ParameterDefinitionCollection (this);
			m_hasThis = hasThis;
			m_explicitThis = explicitThis;
			m_callConv = callConv;
		}

		internal MethodReference (string name) : base (name)
		{
			m_returnType = new MethodReturnType (null);
		}

		public MethodReference (string name,
			TypeReference declaringType, TypeReference returnType,
			bool hasThis, bool explicitThis, MethodCallingConvention callConv) :
			this (name, hasThis, explicitThis, callConv)
		{
			this.DeclaringType = declaringType;
			this.ReturnType.ReturnType = returnType;
		}

		public virtual void Accept (IReflectionVisitor visitor)
		{
		}

		public override string ToString ()
		{
			StringBuilder sb = new StringBuilder ();
			sb.Append (m_returnType.ReturnType.FullName);
			sb.Append (" ");
			sb.Append (base.ToString ());
			sb.Append ("(");
			for (int i = 0; i < this.Parameters.Count; i++) {
				sb.Append (this.Parameters [i].ParameterType.FullName);
				if (i < this.Parameters.Count - 1)
					sb.Append (",");
			}
			sb.Append (")");
			return sb.ToString ();
		}
	}
}
