/* Copyright (C) 2004 - 2006  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
using System;
using System.Collections;
using System.IO;
using System.Security;

namespace Mono.Xml {

	// convert an XML document into SecurityElement objects
	internal class SecurityParser : SmallXmlParser, SmallXmlParser.IContentHandler {

		private SecurityElement root;

		public SecurityParser () : base ()
		{
			stack = new Stack ();
		}

		public void LoadXml (string xml)
		{
			root = null;
#if CF_1_0
			stack = new Stack ();
#else
			stack.Clear ();
#endif
			Parse (new StringReader (xml), this);
		}

		public SecurityElement ToXml ()
		{
			return root;
		}

		// IContentHandler

		private SecurityElement current;
		private Stack stack;

		public void OnStartParsing (SmallXmlParser parser) {}

		public void OnProcessingInstruction (string name, string text) {}

		public void OnIgnorableWhitespace (string s) {}

		public void OnStartElement (string name, SmallXmlParser.IAttrList attrs)
		{
			SecurityElement newel = new SecurityElement (name);
			if (root == null) {
				root = newel;
				current = newel;
			}
			else {
				SecurityElement parent = (SecurityElement) stack.Peek ();
				parent.AddChild (newel);
			}
			stack.Push (newel);
			current = newel;
			// attributes
			int n = attrs.Length;
			for (int i=0; i < n; i++)
				current.AddAttribute (attrs.GetName (i), attrs.GetValue (i));
		}

		public void OnEndElement (string name)
		{
			current = (SecurityElement) stack.Pop ();
		}

		public void OnChars (string ch)
		{
			current.Text = ch;
		}

		public void OnEndParsing (SmallXmlParser parser) {}
	}
}

